import ctypes
from ctypes import wintypes
import time

user32 = ctypes.WinDLL('user32', use_last_error=True)

INPUT_MOUSE    = 0
INPUT_KEYBOARD = 1
INPUT_HARDWARE = 2

KEYEVENTF_EXTENDEDKEY = 0x0001
KEYEVENTF_KEYUP       = 0x0002
KEYEVENTF_UNICODE     = 0x0004
KEYEVENTF_SCANCODE    = 0x0008


MAPVK_VK_TO_VSC = 0

# msdn.microsoft.com/en-us/library/dd375731 Virtual key Definitions
VK_LBUTTON = 0x01
VK_RBUTTON = 0x02
VK_CANCEL = 0X03
VK_MBUTTON = 0X04
VK_XBUTTON1 = 0X05
VK_XBUTTON2 = 0X06
VK_BACK = 0X08
VK_TAB  = 0x09
VK_CLEAR = 0x0C
VK_RETURN = 0x0D
VK_SHIFT = 0x10
VK_CONTROL = 0x11
VK_MENU = 0x12 #ALT KEY
VK_PAUSE = 0x13
VK_CAPITAL = 0x14

VK_ESCAPE = 0x1B

VK_SPACE = 0x20
VK_PRIOR = 0x21 #Page UP
VK_NEXT = 0x22  #Page DOWN
VK_END = 0x23
VK_HOME = 0x24
VK_LEFT = 0x25
VK_UP = 0x26
VK_RIGHT = 0x27
VK_DOWN = 0x28
VK_SELECT = 0x29
VK_PRINT = 0x2A
VK_EXECUTE = 0x2B
VK_SNAPSHOT = 0x2C
VK_INSERT = 0x2D
VK_DELETE = 0x2E
VK_HELP = 0x2F

VK_NUM = [code for code in range(0x30,0x3A)]
alpha = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
VK_ALPHA = { alpha[i-0x41]: i for i in range(0x41,0x5B)}

VK_LWIN = 0x5B #left windows key
LK_RWIN = 0x5C #right windows key
VK_APPS = 0x5D #applications key
VK_SLEEP = 0x5F #computer sleep key
VK_NUMPAD = [code for code in range(0x60,0x6A)]
VK_MULTIPLY = 0x6A
VK_ADD = 0x6B
VK_SEPARATOR = 0x6C
VK_SUBSTRACT = 0x6D
VK_DECIMAL = 0x6E
VK_DIVIDE = 0x6F
VK_F = [0] + [code for code in range(0x70,0x87)]
VK_NUMLOCK = 0x90
VK_SCROLL = 0x91
VL_LSHIFT = 0xA0 #left shift
VK_RSHIFT = 0xA1 #right shift
VK_LCONTROL = 0xA2 #left control
VK_RCONTROL = 0xA3 #right control
VK_LMENU = 0xA4 # left Alt
VK_RMENU = 0xA5 # right Alt
VK_BROWSER_BACK = 0xA6
VK_BROWSER_FORWARD = 0xA7
VK_BROWSER_REFRESH = 0xA8
VK_BROWSER_STOP = 0xA9
VK_BROWSER_SEARCH = 0xAA
VK_BROWSER_FAVORITES = 0xAB
VK_BROWSER_HOME = 0xAC
VK_VOLUME_MUTE = 0xAD
VK_VOLUME_DOWN = 0xAE
VK_VOLUME_UP = 0xAF
VK_MEDIA_NEXT_TRACK = 0xB0
VK_MEDIA_PREV_TRACK = 0xB0
VK_MEDIA_STOP = 0xB2
VK_MEDIA_PLAY_PAUSE = 0xB3
VK_LAUNCH_MAIL = 0xB4
VK_LAUNCH_MEDIA_SELECT = 0xB5
VK_OEM_1 = 0xBA # ';:' key
VK_OEM_PLUS = 0XBB # '=+' KEY
VK_OEM_COMMA = 0xBC # ',<' key
VK_OEM_MINUS = 0xBD # '-_' key
VK_OEM_PERIOD = 0xBE # '.>' key
VK_OEM_2 = 0xBF # '/?' key
VK_OEM_3 = 0xC0 # '`~' key
VK_OEM_4 = 0xDB # '[{' key
VK_OEM_5 = 0xDC # '\|' key
VK_OEM_6 = 0xDD # ']}' key
VK_OEM_7 = 0xDE # ''"' key


# C struct definitions

wintypes.ULONG_PTR = wintypes.WPARAM

class MOUSEINPUT(ctypes.Structure):
    _fields_ = (("dx",          wintypes.LONG),
                ("dy",          wintypes.LONG),
                ("mouseData",   wintypes.DWORD),
                ("dwFlags",     wintypes.DWORD),
                ("time",        wintypes.DWORD),
                ("dwExtraInfo", wintypes.ULONG_PTR))

class KEYBDINPUT(ctypes.Structure):
    _fields_ = (("wVk",         wintypes.WORD),
                ("wScan",       wintypes.WORD),
                ("dwFlags",     wintypes.DWORD),
                ("time",        wintypes.DWORD),
                ("dwExtraInfo", wintypes.ULONG_PTR))

    def __init__(self, *args, **kwds):
        super(KEYBDINPUT, self).__init__(*args, **kwds)
        # some programs use the scan code even if KEYEVENTF_SCANCODE
        # isn't set in dwFflags, so attempt to map the correct code.
        if not self.dwFlags & KEYEVENTF_UNICODE:
            self.wScan = user32.MapVirtualKeyExW(self.wVk,
                                                 MAPVK_VK_TO_VSC, 0)

class HARDWAREINPUT(ctypes.Structure):
    _fields_ = (("uMsg",    wintypes.DWORD),
                ("wParamL", wintypes.WORD),
                ("wParamH", wintypes.WORD))

class INPUT(ctypes.Structure):
    class _INPUT(ctypes.Union):
        _fields_ = (("ki", KEYBDINPUT),
                    ("mi", MOUSEINPUT),
                    ("hi", HARDWAREINPUT))
    _anonymous_ = ("_input",)
    _fields_ = (("type",   wintypes.DWORD),
                ("_input", _INPUT))

LPINPUT = ctypes.POINTER(INPUT)

def _check_count(result, func, args):
    if result == 0:
        raise ctypes.WinError(ctypes.get_last_error())
    return args

user32.SendInput.errcheck = _check_count
user32.SendInput.argtypes = (wintypes.UINT, # nInputs
                             LPINPUT,       # pInputs
                             ctypes.c_int)  # cbSize

# Functions

def PressKey(hexKeyCode):
    x = INPUT(type=INPUT_KEYBOARD,
              ki=KEYBDINPUT(wVk=hexKeyCode))
    user32.SendInput(1, ctypes.byref(x), ctypes.sizeof(x))

def ReleaseKey(hexKeyCode):
    x = INPUT(type=INPUT_KEYBOARD,
              ki=KEYBDINPUT(wVk=hexKeyCode,
                            dwFlags=KEYEVENTF_KEYUP))
    user32.SendInput(1, ctypes.byref(x), ctypes.sizeof(x))


def GenVK(keyname):
    """
    Returns the VK keycode for a specific key name
    """
    keyname = keyname.upper()

    if len(keyname)==1:
        if keyname.isalpha():
            return VK_ALPHA[keyname]
        elif keyname.isdigit():
            return VK_NUM[int(keyname)]
        else:
            if keyname in ';:':
                return VK_OEM_1
            elif keyname in '=+':
                return VK_OEM_PLUS
            elif keyname in ',<':
                return VK_OEM_COMMA
            elif keyname in '-_':
                return VK_OEM_MINUS
            elif keyname in '.>':
                return VK_OEM_PERIOD
            elif keyname in '/?':
                return VK_OEM_2
            elif keyname in '`~':
                return VK_OEM_3
            elif keyname in '[{':
                return VK_OEM_4
            elif keyname in '\|':
                return VK_OEM_5
            elif keyname in ']}':
                return VK_OEM_6
            elif keyname in '\'"':
                return VK_OEM_7

    elif len(keyname)==2:
        if keyname[0]=="N":
            if keyname[1].isdigit():
                return VK_NUMPAD[int(keyname[1])]
            elif keyname[1]=="*":
                return VK_MULTIPLY
            elif keyname[1]=="+":
                return VK_ADD
            elif keyname[1]=="-":
                return VK_SUBTRACT
            elif keyname[1]=="/":
                return VK_DIVIDE
            elif keyname[1]==".":
                return VK_DECIMAL
        elif keyname[0]=="F":
            if keyname[1].isdigit():
                return VK_F[int(keyname[1])]
        elif keyname=="UP":
            return VK_UP
            
    elif len(keyname)==3:
        if keyname[0]=="F":
            if keyname[1:].isdigit():
                return VK_F[int(keyname[1:])]
        elif keyname=="ESC":
            return VK_ESCAPE
        elif keyname=="ALT":
            return VK_MENU
        elif keyname=="END":
            return VK_END
        elif keyname=="INS":
            return VK_INSERT
        elif keyname=="DEL":
            return VK_DELETE
        elif keyname=="TAB":
            return VK_TAB
        elif keyname=="WIN":
            return VK_LWIN
    elif len(keyname)==4:
        if keyname=="DOWN":
            return VK_DOWN
        elif keyname=="LEFT":
            return VK_LEFT
        elif keyname=="CTRL":
            return VK_CONTROL
        elif keyname=="HOME":
            return VK_HOME
        elif keyname=="PGUP":
            return VK_PRIOR
        elif keyname=="PGDN":
            return VK_NEXT
        elif keyname=="LALT":
            return VK_LMENU
        elif keyname=="RALT":
            return VK_RMENU
        elif keyname=="LWIN":
            return VK_LWIN
        elif keyname=="RWIN":
            return VK_RWIN
        elif keyname=="CAPS":
            return VK_CAPITAL
        
    elif len(keyname)==5:
        if keyname=="SHIFT":
            return VK_SHIFT
        elif keyname=="RIGHT":
            return VK_RIGHT
        elif keyname=="SPACE":
            return VK_SPACE
        elif keyname=="ENTER":
            return VK_RETURN
        elif keyname=="PAUSE":
            return VK_PAUSE
        elif keyname=="LCTRL":
            return VK_LCONTROL
        elif keyname=="RCTRL":
            return VK_RCONTROL
    elif len(keyname)==6:
        if keyname=="LSHIFT":
            return VK_LSHIFT
        elif keyname=="RSHIFT":
            return VR_RSHIFT
        elif keyname=="BCKSPC":
            return VK_BACK
        elif keyname=="NUMLCK":
            return VK_NUMLOCK
    return 0

import sys

key_alias = ["UP","DOWN","LEFT","RIGHT","SPACE"]

if len(sys.argv)<(1+1+5+1) and len(sys.argv)>2:
    key_alias = [sys.argv[i] for i in range(2,len(sys.argv))]

FRONTKEYS = key_alias[0].split("+")
BACKKEYS = key_alias[1].split("+")
LEFTKEYS = key_alias[2].split("+")
RIGHTKEYS = key_alias[3].split("+")
CENTERKEYS = key_alias[4].split("+")
    
                               
def FrontKeyPressAction():
    TEMPKEYS = list(FRONTKEYS)
    for key in TEMPKEYS:
        print("Pressed: "+key)
        PressKey(GenVK(key))
        time.sleep(0.01)

def FrontKeyReleaseAction():
    TEMPKEYS = list(FRONTKEYS)
    TEMPKEYS.reverse()
    for key in TEMPKEYS:
        print("Released: "+key)
        ReleaseKey(GenVK(key))
        time.sleep(0.01)

def BackKeyPressAction():
    TEMPKEYS = list(BACKKEYS)
    for key in TEMPKEYS:
        print("Pressed: "+key)
        PressKey(GenVK(key))
        time.sleep(0.01)

def BackKeyReleaseAction():
    TEMPKEYS = list(BACKKEYS)
    TEMPKEYS.reverse()
    for key in TEMPKEYS:
        print("Released: "+key)
        ReleaseKey(GenVK(key))
        time.sleep(0.01)

def LeftKeyPressAction():
    TEMPKEYS = list(LEFTKEYS)
    for key in TEMPKEYS:
        print("Pressed: "+key)
        PressKey(GenVK(key))
        time.sleep(0.01)

def LeftKeyReleaseAction():
    TEMPKEYS = list(LEFTKEYS)
    TEMPKEYS.reverse()
    for key in TEMPKEYS:
        print("Released: "+key)
        ReleaseKey(GenVK(key))
        time.sleep(0.01)

def RightKeyPressAction():
    TEMPKEYS = list(RIGHTKEYS)
    for key in TEMPKEYS:
        print("Pressed: "+key)
        PressKey(GenVK(key))
        time.sleep(0.01)

def RightKeyReleaseAction():
    TEMPKEYS = list(RIGHTKEYS)
    TEMPKEYS.reverse()
    for key in TEMPKEYS:
        print("Released: "+key)
        ReleaseKey(GenVK(key))
        time.sleep(0.01)

def CenterKeyPressAction():
    TEMPKEYS = list(CENTERKEYS)
    for key in TEMPKEYS:
        print("Pressed: "+key)
        PressKey(GenVK(key))
        time.sleep(0.01)

def CenterKeyReleaseAction():
    TEMPKEYS = list(CENTERKEYS)
    TEMPKEYS.reverse()
    for key in TEMPKEYS:
        print("Released: "+key)
        ReleaseKey(GenVK(key))
        time.sleep(0.01)

import serial
port = sys.argv[1]
baud = 115200


if __name__ == "__main__":
    s = serial.Serial(port,baud,timeout=0)
    while 1:
        d = s.read()
        if d:
            d = d.decode('utf-8')
            print(d)
            if d=='F':
                FrontKeyPressAction()
            elif d=='f':
                FrontKeyReleaseAction()
            elif d=='B':
                BackKeyPressAction()
            elif d=='b':
                BackKeyReleaseAction()
            elif d=='L':
                LeftKeyPressAction()
            elif d=='l':
                LeftKeyReleaseAction()
            elif d=='R':
                RightKeyPressAction()
            elif d=='r':
                RightKeyReleaseAction()
            elif d=='S':
                CenterKeyPressAction()
            elif d=='s':
                CenterKeyReleaseAction()
            
